import 'dart:io';

import 'package:bot_toast/bot_toast.dart';
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:ocg_app/src/widgets/custom_bottom_sheet.dart';

class AppUIHelper {
  final BuildContext context;

  AppUIHelper._(this.context);

  factory AppUIHelper.of(BuildContext context) => AppUIHelper._(context);

  String getPlaceholderImage(String name) {
    if (name == null) {
      return null;
    }
    return "https://ui-avatars.com/api/?name=$name&background=${intToRGB(getHexCode(name))}&color=ffffff&size=512";
  }

  int getHexCode(String value) {
    // java String#hashCode
    var hash = 0;
    for (var i = 0; i < value.length; i++) {
      hash = value.codeUnitAt(i) + ((hash << 5) - hash);
    }
    return hash;
  }

  String intToRGB(int i) {
    var c = (i & 0x00FFFFFF).toRadixString(16).toUpperCase();

    return "00000".substring(0, 6 - c.length) + c;
  }

  Future<void> showErrorDialog(dynamic exception) {
    return showAlertDialog(
      'Error',
      exception.toString(),
    );
  }

  Future<void> showAlertDialog(
    String title,
    String message,
  ) {
    return showDialog(
      context: context,
      builder: (context) => AlertDialog(
        title: Text(title),
        content: message != null ? Text(message) : null,
        actions: <Widget>[
          FlatButton(
            child: Text('OK'),
            onPressed: () => Navigator.of(context).pop(),
          ),
        ],
      ),
    );
  }

  Future<void> showLoadingDialog(
    String title,
  ) {
    return showDialog(
      context: context,
      builder: (context) => AlertDialog(
          title: Platform.isIOS ? Text("$title") : null,
          shape:
              RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
          content: Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              CircularProgressIndicator(),
              SizedBox(
                width: 12,
              ),
              Text("$title")
            ],
          )),
    );
  }

  Future<int> showBottomSheet<T>(
      {String title, List<T> options, Function(T option) itemBuilder}) async {
    return await showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        builder: (context) => CustomBottomSheet<T>(
            title: title,
            options: options,
            itemBuilder: itemBuilder,
            onSelected: (index) {
              Navigator.of(context).pop(index);
            }));
  }

  Future<void> showToast({@required String text}) async {
    BotToast.showText(text: text);
  }

  Function showLoadingToast(String title) {
    var cancelFun = BotToast.showCustomLoading(
        toastBuilder: (context) => Center(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  CircularProgressIndicator(
                    valueColor: AlwaysStoppedAnimation<Color>(Colors.white),
                  ),
                  SizedBox(
                    height: 4,
                  ),
                  Text(
                    "$title",
                    style: TextStyle(color: Colors.white),
                  )
                ],
              ),
            ));
    return cancelFun;
  }

  Future<String> showModalDialog(BuildContext context, List<String> values,
      {String title}) async {
    var result;
    if (Platform.isIOS) {
      result = await showCupertinoModalPopup(
          useRootNavigator: true,
          context: context,
          builder: (context) => CupertinoActionSheet(
                actions: values
                    .map((val) => CupertinoActionSheetAction(
                          child: Text(
                            "$val",
                          ),
                          onPressed: () {
                            Navigator.of(context).pop(val);
                          },
                        ))
                    .toList(),
                cancelButton: CupertinoActionSheetAction(
                  child: Text("Cancel"),
                  isDestructiveAction: true,
                  onPressed: () {
                    Navigator.of(context).pop("Cancel");
                  },
                ),
              ));
      return result;
    } else {
      result = await showModalBottomSheet(
          useRootNavigator: true,
          context: context,
          builder: (context) => Column(
                mainAxisSize: MainAxisSize.min,
                children: values
                    .map((val) => ListTile(
                          title: Text("$val"),
                          onTap: () {
                            Navigator.of(context).pop(val);
                          },
                          trailing: Icon(Icons.chevron_right),
                        ))
                    .toList(),
              ));
    }
    return result;
  }
}
